import 'package:flutter/foundation.dart';
import 'package:ocg_app/ocg_app.dart';
import 'package:sembast/sembast.dart';
import 'package:sembast/sembast_io.dart';
import 'package:path_provider/path_provider.dart';
import 'package:sembast_web/sembast_web.dart';

abstract class ProfileDatabase {
  Future<AppResponse<String>> getCurrent();
  Future<AppResponse<bool>> deleteCurrent();
  Future<AppResponse<bool>> saveCurrent(String id);
}

class ProfileTestDatabase extends ProfileDatabase {
  final StoreRef _store = StoreRef.main();
  final String record;

  Database _database;

  ProfileTestDatabase(this.record) : this._database = null;

  Future<void> _open() async {
    if (_database != null) {
      return;
    }
    String dbPath;
    if (kIsWeb) {
      dbPath = "/profiles.db";
      _database = await databaseFactoryWeb.openDatabase(dbPath);
    } else {
      final DatabaseFactory _databaseFactory = databaseFactoryIo;
      dbPath = (await getApplicationSupportDirectory()).path + "/profiles.db";
      _database = await _databaseFactory.openDatabase(dbPath);
    }
  }

  @override
  Future<AppResponse<bool>> deleteCurrent() async {
    await _open();
    try {
      await _store.record(record).delete(
            _database,
          );
      return AppResponse.named(data: true);
    } catch (e) {
      return AppResponse.named(error: e.toString());
    }
  }

  @override
  Future<AppResponse<String>> getCurrent() async {
    await _open();
    try {
      var value = await _store.record(record).get(
            _database,
          );
      if (value != null && value is String)
        return AppResponse.named(data: value);
      return AppResponse.named(error: "Not found");
    } catch (e) {
      return AppResponse.named(error: e.toString());
    }
  }

  @override
  Future<AppResponse<bool>> saveCurrent(String id) async {
    await _open();
    try {
      var key = await _store.record(record).put(_database, id);
      debugPrint("Key $key Id $id");

      if (key != null) return AppResponse.named(data: true);
      return AppResponse.named(error: "Not Saved");
    } catch (e) {
      debugPrint("Error $e");
      return AppResponse.named(error: e.toString());
    }
  }
}
