import 'package:equatable/equatable.dart';
import 'package:json_annotation/json_annotation.dart';

part 'profile.g.dart';

@JsonSerializable(explicitToJson: true)
class Profile extends Equatable {
  final String id;
  final String userId;
  final String name;
  final String profilePic;
  final String mobile;
  final String type;
  final List<String> followers;
  final List<String> following;
  final String username;

  Profile(
    this.id,
    this.userId,
    this.name,
    this.username,
    this.profilePic,
    this.mobile,
    this.type,
    this.followers,
    this.following,
  );

  Profile.named({
    this.id,
    this.userId,
    this.name,
    this.profilePic,
    this.mobile,
    this.type,
    this.followers,
    this.following,
    this.username,
  });

  Map<String, dynamic> toJson() => _$ProfileToJson(this);

  factory Profile.fromJson(json) => _$ProfileFromJson(json);

  @override
  List<Object> get props => [id];

  Profile copyProfile({
    String id,
    String userId,
    String name,
    String profilePic,
    String mobile,
    String username,
    Map<String, SocialProvider> providers,
    String type,
    List<String> following,
    List<String> followers,
  }) {
    return Profile.named(
        id: id ?? this.id,
        userId: userId ?? this.userId,
        name: name ?? this.name,
        profilePic: profilePic ?? this.profilePic,
        type: type ?? this.type,
        mobile: mobile ?? this.mobile,
        username: username ?? this.username,
        followers: followers ?? this.followers,
        following: following ?? this.following);
  }
}

@JsonSerializable(explicitToJson: true)
class SocialProvider {
  final Map metadata;
  final String provider;
  final String id;

  SocialProvider(this.metadata, this.provider, this.id);

  SocialProvider.named({this.metadata, this.provider, this.id});

  Map<String, dynamic> toJson() => _$SocialProviderToJson(this);

  factory SocialProvider.fromJson(json) => _$SocialProviderFromJson(json);
}

@JsonSerializable(explicitToJson: true)
class ProfileFollowing {
  final String from;
  final String to;
  final DateTime recordedAt;

  ProfileFollowing(this.from, this.to, this.recordedAt);

  ProfileFollowing.named({
    this.from,
    this.to,
    this.recordedAt,
  });

  factory ProfileFollowing.fromJson(json) => _$ProfileFollowingFromJson(json);

  Map<String, dynamic> toJson() => _$ProfileFollowingToJson(this);
}
